;;; mypaedia.el --- supplement file for $B!X%^%$%Z%G%#%"#9#9!Y(B
;; 2000/04/11
;;
;; Copyright (C) 2000 Keisuke Nishida <kxn30@po.cwru.edu>
;; Copyright (C) 2000 Kazuhiko Shiozaki <kazuhiko@ring.gr.jp>
;; Copyright (C) 2000 Kazuyoshi KOREEDA <k_koreed@d2.dion.ne.jp>

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software Foundation,
;; Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

;;; Usage:

;; ~/.lookup $B$G!"(B
;;   (setq lookup-search-agents '((ndeb "/usr/share/dict/mypaedia")))
;;   (setq lookup-package-directory "/usr/share/dict/package")
;;   (setq mypaedia-data-directory
;;         "/mnt/cdrom/INDEX") ; BH.DAT, CH.DAT, MAP.DAT, PICT.DAT $B$N$"$k>l=j(B
;;   (setq mypaedia-av-directory
;;         "/mnt/cdrom/DATA") ; *.MID, *.WAV, *.AVI $B$N$"$k>l=j(B
;;   (setq play-midi-process "timidity")
;;   (setq play-wav-process "soxplay")
;;   (setq play-avi-process "xanim")
;;   (setq display-image-process "display")
;;   (lookup-use-package "ndeb+/usr/share/dict/mypaedia" "mypaedia")
;; 
;; $B$N$h$&$K@_Dj$7$F!"(Blookup-package-directory $B$K$3$N%U%!%$%k$rJ]B8$7$F(B
;; $B2<$5$$!#(Bxemacs $B$G$O2hA|$r%$%s%i%$%sI=<($7$^$9$,!"%-%c%C%7%e$r8z$+$;(B
;; $B$F$J$$$N$G!"2hA|$r:FI=<($7$?$$$H$-$K$O(B `g' $B$G%j%m!<%I$9$kI,MW$,$"$j(B
;; $B$^$9!#(BMIDI, WAV, AVI $B$N$&$A:F@8$7$?$/$J$$$b$N$K$D$$$F$O!"(B
;;   (setq play-wav-process nil)
;; $B$N$h$&$K@_Dj$7$F$/$@$5$$!#(B
;;
;; $B$J$*!"(BAPEL $B$,I,MW$G$9!#$"$i$+$8$a%$%s%9%H!<%k$7$F$*$$$F$/$@$5$$!#(B
;; http://www.m17n.org/APEL/
;; ftp://ftp.m17n.org/pub/mule/apel/

;; $B;29M(B URL
;;
;; TiMidity++ -- MIDI $B:F@8(B
;; http://www.goice.co.jp/member/mo/timidity/
;;
;; sox -- WAV $B:F@8(B
;; http://home.sprynet.com/~cbagwell/sox.html
;;
;; xanim -- AVI $B:F@8(B (cinepak $BMQ$N(B dll $B$bI,MW(B)
;; http://xanim.va.pubnix.com/home.html
;;
;; ImageMagick -- $B3F<o2hA|$NI=<((B
;; http://www.wizards.dupont.com/cristy/ImageMagick.html

;;; Code:

(require 'lookup-package)
(require 'lookup-content)
(require 'poem)

(defvar mypaedia-data-directory "/mnt/cdrom/INDEX"
  "BH.DAT, CH.DAT, MAP.DAT, PICT.DAT $B$N$"$k>l=j!#(B")
(defvar mypaedia-av-directory "/mnt/cdrom/DATA"
  "*.MID, *.WAV, *.AVI $B$N$"$k>l=j!#(B")
(defvar play-midi-process "timidity"
  "MIDI $B$r:F@8$9$k%W%m%;%9L>!#(Bnil $B$J$i:F@8$7$J$$!#(B")
(defvar play-wav-process "soxplay"
  "WAV $B$r:F@8$9$k%W%m%;%9L>!#(Bnil $B$J$i:F@8$7$J$$!#(B")
(defvar play-avi-process "xanim"
  "AVI $B$r:F@8$9$k%W%m%;%9L>!#(Bnil $B$J$i:F@8$7$J$$!#(B")
(defvar display-image-process "display"
  "$B2hA|(B $B$rI=<($9$k%W%m%;%9L>!#(Bnil $B$J$iI=<($7$J$$!#(B")
(defvar mypaedia-sound-without-notice nil
  "t $B$J$i8!:w$HF1;~$K2;@<$r:F@8$9$k!#(B")
(defvar mypaedia-video-without-notice nil
  "t $B$J$i8!:w$HF1;~$KF02h$r:F@8$9$k!#(B")
(defvar display-image-inline t
  "nil $B$J$i2hA|$r%$%s%i%$%sI=<($7$J$$!#(B")

(defvar mypaedia-tmp-dir temporary-file-directory "$B0l;~%U%!%$%k$N:n@.>l=j(B")

(defconst mypaedia-arranges
  '(lookup-arrange-gaijis
    mypaedia-arrange-midi
    mypaedia-arrange-wav
    mypaedia-arrange-avi
    mypaedia-arrange-images
    lookup-arrange-references
    lookup-arrange-default-headings
    lookup-arrange-fill-lines))

(defvar mypaedia-dictionary-options
  (list (cons ':arranges mypaedia-arranges)))

(defvar mypaedia-process-file-alist '())

;; 
;; $B85$+$i$"$k(B lookup-content-follow-link$B$N3HD%(B
;; 
(unless (fboundp 'lookup-content-follow-link:old)
  (fset 'lookup-content-follow-link:old
	(symbol-function 'lookup-content-follow-link))
  (defun lookup-content-follow-link ()
    (interactive)
    (let ((action (get-text-property (point) 'action)))
      (if action 
	  (funcall action (point))
	(lookup-content-follow-link:old)))))

(defun mypaedia-string-to-number (string)
  (let ((len (length string))
	(number 0) (i 0) c)
    (while (< i len)
      (setq number (* number 16))
      (setq c (aref string i))
      (cond
       ((and (<= ?0 c) (<= c ?9)) (setq number (+ number (- c ?0))))
       ((and (<= ?a c) (<= c ?f)) (setq number (+ number (- c ?a -10))))
       ((and (<= ?A c) (<= c ?F)) (setq number (+ number (- c ?A -10))))
       (t (setq i len)))
      (setq i (1+ i)))
    number))

(defun mypaedia-arrange-images (entry)
  (while (re-search-forward
	  "<image=\\([^:>]+\\):\\([^:]+\\):\\([^>]+\\)>" nil t)
    (let ((file (concat (match-string 1) ".DAT"))
	  (offset (mypaedia-string-to-number (match-string 2)))
	  (length (mypaedia-string-to-number (match-string 3)))
	  (start (match-beginning 0))
	  (end (match-end 0)))
	  ;; Find data file.
      (if (file-exists-p (expand-file-name file mypaedia-data-directory))
	  (setq file (expand-file-name file mypaedia-data-directory)))
      (if (and display-image-inline
	       (featurep 'xemacs)
	       (eq window-system 'x))
	  (progn
	    (replace-match "\n")
	    (insert-file-contents-internal file nil offset (+ offset length)))
	(replace-match "$B"*(B[$B2hA|(B]")
	(add-text-properties start 
			     (+ (length "$B"*(B[$B2hA|(B]") start)
			     (list 'action 'mypaedia-display-image
				   'file  file
				   'offset offset
				   'mouse-face 'highlight
				   'face 'lookup-reference-face
				   'size   length))))))
;;
;; $B30It%W%m%;%9$rMxMQ$7$?%$%a!<%8$NI=<((B
;;
(defun mypaedia-display-image (pos)
  (let* ((file (get-text-property pos 'file))
	 (offset (get-text-property pos 'offset))
	 (size (get-text-property pos 'size))
	 (tmp-image-file (mypaedia-write-tmp-file 
			  file offset (+ offset size))))
    (if display-image-process
	(mypaedia-start-process display-image-process nil tmp-image-file t))))
;;
;; $B30It%W%m%;%9$N8F=P$7(B
;; 
(defun mypaedia-start-process (program options file &optional delete-file)
    (message "Starting %s ..." program)
    (let ((pro (apply (function start-process)
		      (format "*mypaedia %s*" program)
		      nil
		      program
		      (append options (list file)))))
      	(message "Sending %s ... done" program)
	(set-process-sentinel pro 'mypaedia-start-process-sentinel)
	(setq mypaedia-process-file-alist 
		(cons (cons pro file) 
		      (if delete-file 
			  mypaedia-process-file-alist
			nil)))))
;;
;; $B%W%m%;%9$N>uBV$,JQ99$5$l$?$H$-$K%U%!%$%k$r:o=|$9$k!#(B
;;
(defun mypaedia-start-process-sentinel (process event)
  (let ((al (assoc process mypaedia-process-file-alist)))
    (and (cdr al) (delete-file (cdr al)))
    (setq mypaedia-process-file-alist 
	  (delete al mypaedia-process-file-alist))))
;;
;; $B2hA|%U%!%$%k$N2t$+$iI=<($9$k%U%!%$%k$r@Z$j=P$9!#(B
;;
(defun mypaedia-write-tmp-file (file beg end)
  (let ((tmp-file (make-temp-name 
		   (expand-file-name "mypaedia" temporary-file-directory))))
    (with-temp-buffer
      (insert-file-contents-as-binary file nil beg end)
      (write-region-as-binary (point-min) (point-max) tmp-file))
;      (set-buffer-file-coding-system 'raw-text)
;      (insert-file-contents-literally file nil beg end)
;      (write-region (point-min) (point-max) tmp-file))
    tmp-file))

;;
;; $B30It%W%m%;%9$rMxMQ$7$?(B MIDI $B$N:F@8(B
;;
(defun mypaedia-arrange-midi (entry)
  (while (re-search-forward "<sound=\\([^>]+mid\\)>" nil t)
    (let ((file (upcase (match-string 1)))
	  (start (match-beginning 0))
	  (end (match-end 0)))
      (if mypaedia-sound-without-notice
	  (mypaedia-play-midi-sub file t))
      (replace-match "$B"*(B[$B2;@<(B]")
      (add-text-properties start 
			   (+ (length "$B"*(B[$B2;@<(B]") start)
			   (list 'action 'mypaedia-play-midi
				 'file file
				 'mouse-face 'highlight
				 'face 'lookup-reference-face)))))

(defun mypaedia-play-midi-sub (file &optional bulk)
  (if play-midi-process
      (let ((midi-file (expand-file-name file mypaedia-av-directory)))
	(if (not (file-exists-p midi-file))
	    (or bulk (error "%s: not found." midi-file))
	  (if (get-process "lookup-play-midi")
	      (delete-process "lookup-play-midi")
	    (if (get-process "lookup-play-wav")
		(delete-process "lookup-play-wav")))
	  (start-process "lookup-play-midi" nil play-midi-process midi-file))
	)))

(defun mypaedia-play-midi (pos)
  (mypaedia-play-midi-sub (get-text-property pos 'file)))

;;
;; $B30It%W%m%;%9$rMxMQ$7$?(B WAV $B$N:F@8(B
;;
(defun mypaedia-arrange-wav (entry)
  (while (re-search-forward "<sound=\\([^>]+wav\\)>" nil t)
    (let ((file (upcase (match-string 1)))
	  (start (match-beginning 0))
	  (end (match-end 0)))
      (if mypaedia-sound-without-notice
	  (mypaedia-play-wav-sub file t))
      (replace-match "$B"*(B[$B2;@<(B]")
      (add-text-properties start 
			   (+ (length "$B"*(B[$B2;@<(B]") start)
			   (list 'action 'mypaedia-play-wav
				 'file file
				 'mouse-face 'highlight
				 'face 'lookup-reference-face)))))

(defun mypaedia-play-wav-sub (file &optional bulk)
  (if play-wav-process
      (let ((wav-file (expand-file-name file mypaedia-av-directory)))
	(if (not (file-exists-p wav-file))
	    (or bulk (error "%s: not found." wav-file))
	  (if (get-process "lookup-play-midi")
	      (delete-process "lookup-play-midi")
	    (if (get-process "lookup-play-wav")
		(delete-process "lookup-play-wav")))
	  (start-process "lookup-play-wav" nil play-wav-process wav-file)))))

(defun mypaedia-play-wav (pos)
  (mypaedia-play-wav-sub (get-text-property pos 'file)))

;;
;; $B30It%W%m%;%9$rMxMQ$7$?(B AVI $B$N:F@8(B
;;
(defun mypaedia-arrange-avi (entry)
  (while (re-search-forward "<video=\\([^>]+avi\\)>" nil t)
    (let ((file (upcase (match-string 1)))
	  (start (match-beginning 0))
	  (end (match-end 0)))
      (if mypaedia-video-without-notice
	  (mypaedia-play-avi-sub file t))
      (replace-match "$B"*(B[$BF02h(B]")
      (add-text-properties start 
			   (+ (length "$B"*(B[$BF02h(B]") start)
			   (list 'action 'mypaedia-play-avi
				 'file  file
				 'mouse-face 'highlight
				 'face 'lookup-reference-face)))))

(defun mypaedia-play-avi-sub (file &optional bulk)
  (if play-avi-process
      (let ((avi-file (expand-file-name file mypaedia-av-directory)))
	(if (not (file-exists-p avi-file))
	    (or bulk (error "%s: not found." avi-file))
	  (if (get-process "lookup-play-midi")
	      (delete-process "lookup-play-midi"))
	  (start-process "lookup-play-avi" nil play-avi-process avi-file)))))

(defun mypaedia-play-avi (pos)
  (mypaedia-play-avi-sub (get-text-property pos 'file)))
  
(setq lookup-package-dictionary-options-alist
      (list (cons (cond ((eq lookup-package-agent 'ndtp) "MYPAEDIA/MYPAEDIA")
			((eq lookup-package-agent 'ndeb) "MYPAEDIA"))
		  mypaedia-dictionary-options)))

;;; mypaedia.el ends here
